﻿using ProjectTank.Entities;

namespace ProjectTank.Drawnings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>
public class DrawningTank2
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTank2? EntityTank2 { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки танка
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки танка
    /// </summary>
    protected int? _startPosY;
    /// <summary>
    /// Ширина прорисовки танка
    /// </summary>
    private readonly int _drawningTankWidth = 160;
    /// <summary>
    /// Высота прорисовки танка
    /// </summary>
    private readonly int _drawningTankHeight = 94;

    /// <summary>
	/// Координата X объекта
	/// </summary>
	public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningTankWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningTankHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningTank2()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningTank2(int speed, double weight, Color bodyColor) : this()
    {
        EntityTank2 = new EntityTank2(speed, weight, bodyColor);
    }
    /// <summary>
    /// Конструктор для наследования
    /// </summary>
    /// <param name="drawningTankWidth">Ширина прорисовки танка</param>
    /// <param name="drawningTankHeight">Высота прорисовки танка</param>
    protected DrawningTank2(int drawningTankWidth,int drawningTankHeight) : this()
    {
        _drawningTankWidth = drawningTankWidth;
        _drawningTankHeight = drawningTankHeight;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (_drawningTankWidth < width && _drawningTankHeight < height)
        {
            _pictureWidth = width; _pictureHeight = height;

            {
                if (_startPosX.HasValue && _startPosY.HasValue)

                    SetPosition(_startPosX.Value, _startPosY.Value);
            }
            return true;
        }
        return false;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>


    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        if (x < 0 || x + _drawningTankWidth > _pictureWidth || y < 0 || y + _drawningTankHeight > _pictureHeight)
        {
            _startPosX = _pictureWidth - _drawningTankWidth; _startPosY = _pictureHeight - _drawningTankHeight;
        }
        else
        {
            _startPosX = x;
            _startPosY = y;
        }
    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>

    public bool MoveTransport(DirectionType direction)
    {
        if (EntityTank2 == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityTank2.Step > 0)
                {
                    _startPosX -= (int)EntityTank2.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTank2.Step > 0)
                {
                    _startPosY -= (int)EntityTank2.Step;
                }
                return true;

            // вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityTank2.Step + _drawningTankHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityTank2.Step;
                }
                return true;

            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityTank2.Step + _drawningTankWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityTank2.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTank2 == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
       
        //границы танка + гусеницы + пулемёт + башня с оружием
        g.DrawRectangle(pen, _startPosX.Value + 48, _startPosY.Value + 16, 55, 17);
        g.DrawRectangle(pen, _startPosX.Value + 12, _startPosY.Value + 31, 137, 13);

        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 34, 160, 35);
        g.DrawEllipse(pen, _startPosX.Value + 51, _startPosY.Value + 38, 29, 23);
        g.DrawEllipse(pen, _startPosX.Value + 111, _startPosY.Value + 30, 29, 23);
        g.DrawEllipse(pen, _startPosX.Value + 91, _startPosY.Value + 48, 18, 15);
        g.DrawEllipse(pen, _startPosX.Value + 71, _startPosY.Value + 48, 18, 15);
        g.DrawEllipse(pen, _startPosX.Value + 51, _startPosY.Value + 48, 18, 15);

       

        //танк
        Brush br = new SolidBrush(EntityTank2.BodyColor);
        g.FillRectangle(br, _startPosX.Value + 48, _startPosY.Value + 14, 55, 17);
        g.FillRectangle(br, _startPosX.Value + 12, _startPosY.Value + 31, 137, 13);


        Brush brDBlue = new SolidBrush(Color.DarkBlue);
        g.FillEllipse(brDBlue, _startPosX.Value, _startPosY.Value + 34, 160, 35);

        Brush brBlue = new SolidBrush(Color.LightBlue);
        g.FillEllipse(brBlue, _startPosX.Value + 19, _startPosY.Value + 40, 29, 23);
        g.FillEllipse(brBlue, _startPosX.Value + 111, _startPosY.Value + 40, 29, 23);
        g.FillEllipse(brBlue, _startPosX.Value + 91, _startPosY.Value + 48, 18, 15);
        g.FillEllipse(brBlue, _startPosX.Value + 71, _startPosY.Value + 48, 18, 15);
        g.FillEllipse(brBlue, _startPosX.Value + 51, _startPosY.Value + 48, 18, 15);



    }
}
